#!/bin/bash

RED='\033[0;31m'
GREEN='\033[0;32m'
NC='\033[0m' # No Color
CYAN="\033[36m"

check_os() {
  if [[ "$OSTYPE" != "linux-gnu"* ]]; then
    echo -e "${RED}This script only supports Linux distributions.${NC}"
    exit 1
  fi
}

check_architecture() {
  ARCH=$(uname -m)
  case $ARCH in
    x86_64)
      echo -e "${GREEN}Architecture detected: x64${NC}"
      ;;
    aarch64)
      echo -e "${GREEN}Architecture detected: ARM64${NC}"
      ;;
    *)
      echo -e "${RED}Unsupported architecture: $ARCH${NC}"
      exit 1
      ;;
  esac
}

create_directories() {
  mkdir -p logs media
  echo "Created 'logs' and 'media' directories."
}


check_docker_compose_version() {
  if ! docker compose version &> /dev/null; then
    echo -e "${RED}Docker Compose is not installed.${NC}"
    read -p "Would you like to update Docker Compose? [Y/n] " response
    if [[ $response =~ ^[Yy]$ ]]; then
      install_latest_docker_compose
    fi
  fi

  COMPOSE_VERSION=$(docker compose version --short)
  REQUIRED_VERSION="2.0.0"
  if [[ $COMPOSE_VERSION < $REQUIRED_VERSION ]]; then
    echo -e "${RED}Docker Compose version $COMPOSE_VERSION is less than required $REQUIRED_VERSION.${NC}"
    read -p "Would you like to update Docker Compose? [Y/n] " response
    if [[ $response =~ ^[Yy]$ ]]; then
      install_latest_docker_compose
    fi
  fi
}

install_latest_docker_compose() {
  echo "Installing the latest Docker..."
  sudo apt-get update
  sudo apt-get install -y ca-certificates curl gnupg
  sudo install -m 0755 -d /etc/apt/keyrings
  curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/docker.gpg
  sudo chmod a+r /etc/apt/keyrings/docker.gpg

  echo "Adding the Docker repository..."
  echo \
    "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/ubuntu \
    $(. /etc/os-release && echo "$VERSION_CODENAME") stable" | \
    sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
  sudo apt-get update

  echo "Installing Docker Compose..."
  sudo apt-get install -y docker-ce docker-ce-cli containerd.io docker-buildx-plugin docker-compose-plugin

  echo -e "${GREEN}Docker Compose installed successfully.${NC}"
}


check_nvidia_docker() {
  if ! nvidia-container-toolkit -version &> /dev/null; then
    echo -e "${RED}NVIDIA Docker runtime not found.${NC}"
    read -p "Would you like to install NVIDIA Docker runtime? [Y/n] " response
    if [[ $response =~ ^[Yy]$ ]]; then
      echo "Installing NVIDIA Docker runtime..."
      curl -fsSL https://nvidia.github.io/libnvidia-container/gpgkey | sudo gpg --dearmor -o /usr/share/keyrings/nvidia-container-toolkit-keyring.gpg
      curl -s -L https://nvidia.github.io/libnvidia-container/stable/deb/nvidia-container-toolkit.list | \
        sed 's#deb https://#deb [signed-by=/usr/share/keyrings/nvidia-container-toolkit-keyring.gpg] https://#g' | \
        sudo tee /etc/apt/sources.list.d/nvidia-container-toolkit.list

      sudo apt-get update
      sudo apt-get install -y nvidia-container-toolkit
      echo -e "${GREEN}NVIDIA Docker runtime installed successfully.${NC}"

      echo "Configuring the NVIDIA container runtime for Docker..."
      sudo nvidia-ctk runtime configure --runtime=docker

      echo "Restarting the Docker daemon..."
      sudo systemctl restart docker
      echo -e "${GREEN}Docker daemon restarted successfully with NVIDIA runtime.${NC}"
    fi
  fi
}

install_nvidia_regular() {
    echo "Installing NVIDIA drivers for a regular machine..."
    sudo add-apt-repository ppa:graphics-drivers/ppa
    sudo apt update
    sudo apt install software-properties-common
    sudo ubuntu-drivers autoinstall
    echo "Installed Drivers - Please reboot your machine!"
}

check_nvidia_driver() {
    if uname -a | grep -iq 'tegra\|aarch64'; then
        echo "Jetson detected. Make sure you have Jetpack Components installed"
    else
        if ! command -v nvidia-smi &> /dev/null; then
            echo "NVIDIA drivers are not installed."
            read -p "Do you want to install NVIDIA drivers? (y/n) " response
            if [[ "$response" == "y" ]]; then
                install_nvidia_regular
            else
                echo "Installation aborted."
                exit 1
            fi
        else
            echo "NVIDIA drivers are already installed."
        fi
    fi
}

generate_docker_compose() {
  TAG_VERSION="latest"
  SERIAL_NUMBER_VOLUME=""
  SHEM_MEMORY="16gb"

  if uname -a | grep -iq 'tegra\|aarch64'; then
    TAG_VERSION="jetson-latest"
    SERIAL_NUMBER_VOLUME="- /proc/device-tree:/app/device-tree:ro"
    SHEM_MEMORY="8gb"
    echo -e "${GREEN}Jetson detected. Creating docker-compose.yml"
  fi

  cat <<EOF >docker-compose.yml
version: '3'

services:
  web:
    image: devicewise/studio_vi:${TAG_VERSION}
    runtime: nvidia
    command: bash -c "python3 manage.py makemigrations VisualInspectionApp && python3 manage.py migrate VisualInspectionApp && python3 manage.py collectstatic --noinput && daphne VisualInspectionService.asgi:application --port 8000 --bind 0.0.0.0"
    volumes:
      - ./database:/app/database
      - static_volume:/staticfiles
      - ./media:/app/media
      - ./logs:/app/logs
      - /opt/dw/staging:/opt/dw/staging
      ${SERIAL_NUMBER_VOLUME}
    environment:
      - DJANGO_SETTINGS_MODULE=VisualInspectionService.settings
      - NVIDIA_VISIBLE_DEVICES=all
      - OMP_NUM_THREADS=4
      - C_FORCE_ROOT=true
      - PIN_MEMORY=false
    shm_size: '${SHEM_MEMORY}'
    ports:
      - "8000:8000"
    ipc: host
    deploy:
      resources:
        reservations:
          devices:
          - driver: nvidia
            count: all
            capabilities: [gpu]  
    restart: always
            

  nginx:
    image: devicewise/nginx_vi:${TAG_VERSION}
    volumes:
      - static_volume:/staticfiles
      - ./media:/app/media
    ports:
      - "80:80"
      - "3000:443"
    restart: always

  redis:
    image: "redis:alpine"
    ports:
      - "6379:6379"
    restart: always

  celery:
    image: devicewise/studio_vi:${TAG_VERSION}
    runtime: nvidia
    command: ["celery", "-A", "VisualInspectionService", "worker", "--loglevel=DEBUG", "--pool=solo"]
    volumes:
      - ./database:/app/database
      - static_volume:/staticfiles
      - ./media:/app/media
      - ./logs:/app/logs
      - /opt/dw/staging:/opt/dw/staging
      ${SERIAL_NUMBER_VOLUME}
    depends_on:
      - web
      - redis
    environment:
      - DJANGO_SETTINGS_MODULE=VisualInspectionService.settings
      - NVIDIA_VISIBLE_DEVICES=all
      - C_FORCE_ROOT=true
      - PIN_MEMORY=false
    shm_size: '${SHEM_MEMORY}'
    restart: always

  frontend:
    image: devicewise/frontend_vi:latest
    restart: always
volumes:
  static_volume:
EOF
  echo "Generated docker-compose.yml file."
}

build_containers() {
  docker compose build
}

run_containers() {
  docker compose up -d
}

stop_containers() {
  docker compose stop
}

remove_containers() {
  docker compose down
}

check_status() {
  RUNNING_SERVICES=$(docker compose ps --filter "status=running" --services)

  if [[ -z "$RUNNING_SERVICES" ]]; then
    echo -e "${RED}Stopped${NC}"
  else
    echo -e "${GREEN}Running${NC}"
  fi
}

install_devicewise_vi() {
  create_directories
  generate_docker_compose
  INSTALL_LOCATION=$(pwd)  
  echo "Installation of deviceWISE VI completed."
  echo "DeviceWISE VI installed at: $INSTALL_LOCATION" > /tmp/devicewise_vi_install_location.txt
  echo "Install location file created at /tmp/devicewise_vi_install_location.txt"
}

pull_and_run_devicewise_vi() {
  docker compose down
  docker compose pull
  docker compose up -d
  echo "deviceWISE VI updated and started."
}

top_level_menu() {
  clear
  echo -e "${CYAN}      _            _        __        _____ ____  _____  __     _____ "
  echo -e "   __| | _____   _(_) ___ __\ \      / /_ _/ ___|| ____| \ \   / /_ _|"
  echo -e "  / _\` |/ _ \ \ / / |/ __/ _ \ \ /\ / / | |\___ \|  _|    \ \ / / | | "
  echo -e " | (_| |  __/\ V /| | (_|  __/\ V  V /  | | ___) | |___    \ V /  | | "
  echo -e "  \__,_|\___| \_/ |_|\___\___| \_/\_/  |___|____/|_____|    \_/  |___|${NC}\n"

  echo "Please select an option:"
  echo "0) Exit"
  echo "1) Check And Install Prerequisites"
  echo "2) Install deviceWISE VI"
  echo "3) Start deviceWISE VI"
  echo "4) Update and Start deviceWISE VI"
  echo "5) Status of deviceWISE Visual Inspection"
  echo "6) Stop deviceWISE Visual Inspection"
  echo "7) Remove deviceWISE Visual Inspection"
  echo ""
  read -p "Option: " OPTION
  echo ""
}

# menu
main_menu() {
  while true; do
    top_level_menu
    case $OPTION in
      0)
        echo -e "${RED}Exiting...${NC}"
        break
        ;;
      1)
        check_docker_compose_version
        check_nvidia_docker
        check_nvidia_driver
        ;;
      2)
        install_devicewise_vi
        ;;
      3)
        run_containers
        ;;
      4)
        pull_and_run_devicewise_vi
        ;;
      5)
        check_status
        ;;
      6)
        stop_containers
        ;;
      7)
        remove_containers
        ;;
      *)
        echo -e "${RED}Invalid option.${NC}"
        ;;
    esac

    echo ""
    read -n 1 -s -r -p "Press any key to continue"
  done
}



# Script execution starts here
check_os
check_architecture
main_menu
